package com.hzya.frame.plugin.a8bill.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapBuilder;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.ningboBankTreasury.dto.req.SingleTransferReq;
import com.hzya.frame.ningboBankTreasury.dto.req.TTRemitReq;
import com.hzya.frame.ningboBankTreasury.dto.res.AccTransactionRes;
import com.hzya.frame.ningboBankTreasury.enums.CurrencyEnum;
import com.hzya.frame.ningboBankTreasury.enums.PayStatusEnum;
import com.hzya.frame.ningboBankTreasury.enums.TTPayStatusEnum;
import com.hzya.frame.ningboBankTreasury.service.INingboBankTreasuryService;
import com.hzya.frame.plugin.a8bill.dao.IPayBillPluginInitializerDao;
import com.hzya.frame.plugin.a8bill.entity.PayBillEntity;
import com.hzya.frame.plugin.a8bill.service.INingboBankPluginService;
import com.hzya.frame.seeyon.dongj.entity.Formmain0044Entity;
import com.hzya.frame.seeyon.dongj.service.IFormmain0044Service;
import com.hzya.frame.seeyon.util.RestUtil;
import com.hzya.frame.stringutil.StringUtil;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.service.IIntegrationTaskLivingDetailsService;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description 宁波银行插件用
 * @Author xiangerlin
 * @Date 2024/7/25 16:34
 **/
public class NingboBankPluginServiceImpl implements INingboBankPluginService {
    Logger logger = LoggerFactory.getLogger(getClass());
@Autowired
private IFormmain0044Service formmain0044Service;
    @Resource
    private IIntegrationTaskLivingDetailsService taskLivingDetailsService;
    @Autowired
    private IPayBillPluginInitializerDao payBillDao;
    @Autowired
    private RestUtil restUtil;
    @Autowired
    private INingboBankTreasuryService ningboBankTreasuryService;
    @Value("${OA.data_source_code}")
    private String oa_data_source_code;


    /**
     * 支付申请
     * @param hashMaps oa 查到的数据
     * @param pluginId 插件 id
     * @param apiCode api 编码
     * @param interId 重试 id
     */
    @Override
    public void payRequest(List<HashMap<String, Object>> hashMaps, String pluginId, String apiCode,String interId) {
        if(CollectionUtils.isNotEmpty(hashMaps)){
            PayBillEntity oaBillEntity = new PayBillEntity();
            oaBillEntity.setDataSourceCode(oa_data_source_code);

            for (HashMap<String, Object> hashMap : hashMaps) {
                try {
                    JSONObject main = new JSONObject();
                    for(String key:hashMap.keySet()) {
                        main.put(key, hashMap.get(key));
                    }
                    String param = "";
                    //单笔转账
                    if ("8000360000".equals(apiCode)){
                        SingleTransferReq singleTransferReq = JSONObject.parseObject(main.toString(), SingleTransferReq.class);
                        param = JSONObject.toJSONString(singleTransferReq);
                    }else if("8000360003".equals(apiCode)){
                        //TT汇款
                        TTRemitReq ttRemitReq = JSONObject.parseObject(main.toString(), TTRemitReq.class);
                        //货到付款时，要上传附件
                        if ("01".equals(ttRemitReq.getPayMode())){
                            String fileCode = fileUpload(main.getString("id"));
                            ttRemitReq.setFileUploadCodes(Convert.toList(String.class,fileCode));
                        }
                        param = JSONObject.toJSONString(ttRemitReq);
                    }
                    if (StrUtil.isEmpty(param)){
                        param = main.toString();
                    }
                    logger.info("获取到付款单的数据:{}",main.toString());
                    String result = this.executeEsb(param,apiCode,"800036");
                    logger.info("推送宁波银行的返回结果:{}",result);
                    IntegrationTaskLivingDetailsEntity logDetails = new IntegrationTaskLivingDetailsEntity();
                    logDetails.setRootAppPk(main.getString("id"));
                    logDetails.setRootAppBill(main.getString("serialNo"));
                    logDetails.setNewTransmitInfo(result);
                    logDetails.setNewPushDate(new Date());
                    logDetails.setRootAppNewData(JSON.toJSONString(main));
                    logDetails.setPluginId(pluginId);
                    JSONObject resultObj = JSON.parseObject(result);
                    String attribute = resultObj.getString("attribute");
                    boolean flag = resultObj.getBoolean("flag");
                    String subState = "";
                    String fieldName = main.getString("fieldName");
                    String tableName = main.getString("tableName");
                    String payMsg = main.getString("payMsg");
//                    String subMsg = main.getString("subMsg");
                    String str = "";
                    String sync_flag = "";
                    try {
                        if(StrUtil.isEmpty(interId)){// 银行状态 1支付中 2 支付失败 null 未提交支付 0支付成功
                            logDetails.setCreate_time(new Date());
                            logDetails.setModify_time(new Date());
                            if(flag){
                                subState = "1";
                                sync_flag = " ,"+payMsg+" = '支付中'";
                                taskLivingDetailsService.saveLogToSuccess(logDetails);
                            }else{
                                subState = "2";
                                String resMsg = JSONObject.parseObject(attribute).getString("retMsg");
                                sync_flag = " ,"+payMsg+" = '"+resMsg+"'";
                                taskLivingDetailsService.saveLogToFail(logDetails);
                            }
                        }else{
                            logDetails.setId(interId);
                            logDetails.setModify_time(new Date());
                            if(flag){
                                subState = "1";
                                sync_flag = " ,"+payMsg+" = '支付中'";
                                taskLivingDetailsService.saveLogFailToSuccess(logDetails);
                            }else{
                                subState = "2";
                                String resMsg = JSONObject.parseObject(attribute).getString("retMsg");
                                sync_flag = " ,"+payMsg+" = '"+resMsg+"'";
                                taskLivingDetailsService.updateLogFailToSuccess(logDetails);
                            }
                        }

                        str = "update "+tableName+" set " + fieldName + "= '" +subState+"'" +sync_flag+"where id = '"+main.getString("id")+"'";
                    } catch (Exception e) {
                        subState = "N";
                        str = "update "+tableName+" set " + fieldName + "= '" +subState+"'," +payMsg+" = '"+attribute+"' "+"where id = '"+main.getString("id")+"'";
                        logger.info("保存日志失败"+e.getMessage());
                        oaBillEntity.setSql(str);
                        payBillDao.updateStateForId(str,oaBillEntity);
                        e.printStackTrace();
                    }
                    //修改推送状态，防止重复推送
                    oaBillEntity.setSql(str);
                    payBillDao.updateStateForId(str,oaBillEntity);
                }catch (Exception e){
                    logger.info("宁波银行付款接口调用出错:{}",e);
                }
            }
        }
    }

    /**
     * 查询交易结果
     *
     * @param hashMaps
     * @param apiCode
     */
    @Override
    public void queryResult(List<HashMap<String, Object>> hashMaps, String apiCode) {
        if(CollectionUtils.isNotEmpty(hashMaps)){
            PayBillEntity oaBillEntity = new PayBillEntity();
            oaBillEntity.setDataSourceCode(oa_data_source_code);
            for (HashMap<String, Object> hashMap : hashMaps) {
                JSONObject main = new JSONObject();
                for(String key:hashMap.keySet()) {
                    main.put(key, hashMap.get(key));
                }
                String tableName = main.getString("tableName");
                String fieldName = main.getString("fieldName");
                String payDate = main.getString("payDate");
                String payMsg = main.getString("payMsg");
                try {

                    logger.info("获取到付款单状态的数据"+main.toString());
                    JSONObject reMain = new JSONObject();
                    reMain.put("serialNo",main.getString("serialNo"));
                    String result = this.executeEsb(reMain.toString(),apiCode,"800036");
                    logger.info("获取付款状态结果"+result);

                    JSONObject resultObj = JSON.parseObject(result);
                    JSONObject attribute = resultObj.getJSONObject("attribute");
                    boolean flag = resultObj.getBoolean("flag");

                    StringBuffer sb = new StringBuffer();
                    sb.append("update "+tableName);
                    if(flag){
                        JSONObject data = attribute.getJSONObject("Data");
                        JSONObject transferDtl = new JSONObject();
                        String status = "";
                        String msg = "";
                        boolean isFinally = true;
                        if ("8000360001".equals(apiCode)){
                            transferDtl = data.getJSONObject("transferDtl");
                            status = transferDtl.getString("status");
                            msg = PayStatusEnum.getByCode(status);
                            isFinally = PayStatusEnum.isFinally(status);
                        }else if ("8000360007".equals(apiCode)){
                            status = data.getString("transState");
                            msg = TTPayStatusEnum.getByCode(status);
                            isFinally = TTPayStatusEnum.isFinally(status);
                        }
                        if(StrUtil.isEmpty(msg)){
                            msg  ="未知交易状态，请联系管理员查证";
                            sb.append(" set "+payMsg+" = '"+msg+"'");
                            sb.append(" , "+fieldName+" = '2'");
                        }
                        sb.append(" set "+payMsg+" = '"+msg+"'");
                        //是否最终态
                        if(isFinally){
                            sb.append(" , "+fieldName+" = '0'");
                            String payDateStr = DateUtil.now();
                            sb.append(" , "+payDate+" = '"+payDateStr+"'");
                        }
                    }else {
                        sb.append(" set "+payMsg+" = '"+attribute.getString("retMsg")+"'");
                        sb.append(" ,"+fieldName+" = '2'");

                    }
                    sb.append(" where id = '"+main.getString("id")+"'");
                    //修改推送状态，防止重复推送
                    oaBillEntity.setSql(sb.toString());
                    payBillDao.updateStateForId(sb.toString(),oaBillEntity);
                } catch (Exception e) {
                    StringBuffer sb = new StringBuffer();
                    sb.append("update "+tableName );
                    sb.append(" set "+payMsg+" = '获取交易状态失败'");
                    sb.append(" , "+fieldName+" = '2'");
                    sb.append(" where id = '"+main.getString("id")+"'");
                    logger.info("获取交易状态失败，更新交易信息为失败状态，后续不在查询");
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 调用宁波银行esb接口
     *
     * @param params
     * @param apiCode
     * @return
     */
    @Override
    public String executeEsb(String params, String apiCode,String appId) {
        logger.info("请求apiCode:{},参数:{}",apiCode,params);
        Map<String, String> header =  MapBuilder.<String, String>create(true)
                .put("apiCode", apiCode)
                .put("publicKey","ZJYAWb7lhAUTYqekPkU+uHJv1/ObJxb7dT7sD8HPRDGAgyhCe7eDIk+3zDUT+v578prj")
                .put("secretKey","fviZnLBsQUAGF8w8FSOdJi7XlIm/XAZclMxRagDLfTyJFlvnIBF3w66Hrpfzs8cYj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=")
                //.put("appId","800036")
                .put("appId",appId)
                .build();
        String res = HttpRequest.post("http://127.0.0.1:8901/kangarooDataCenterV3/entranceController/externalCallInterface").addHeaders(header).body(params).timeout(50000).execute().body();
        return res;
    }

    /**
     * 保存收款日志到oa底表
     * @param accTransactionResList
     */
    @Override
    public void saveAccLog(List<AccTransactionRes> accTransactionResList) {
        if (CollectionUtils.isNotEmpty(accTransactionResList)){
            //查询已经存在的数据
            Formmain0044Entity formmain0044 = new Formmain0044Entity();
            formmain0044.setStart_date(DateUtil.lastWeek().toDateStr());
            formmain0044.setDataSourceCode(oa_data_source_code);
            List<Formmain0044Entity> formmain0044List = formmain0044Service.querySeridId(formmain0044);
            //过滤已保存的数据
            if (CollectionUtils.isNotEmpty(formmain0044List) && CollectionUtils.isNotEmpty(accTransactionResList)){
                accTransactionResList = accTransactionResList.stream()
                        .filter(acc -> formmain0044List.stream()
                                .noneMatch(f -> acc.getSerialId().equals(f.getField0018())))
                        .collect(Collectors.toList());
            }
            if (CollectionUtils.isNotEmpty(accTransactionResList)){
                for (AccTransactionRes accTransactionRes : accTransactionResList) {
                    String bankAcc = StringUtil.nullConvert(accTransactionRes.getBankAcc());
                    String bankName = StringUtil.nullConvert(accTransactionRes.getBankName());
                    String accName = StringUtil.nullConvert(accTransactionRes.getAccName());
                    String oppAccNo = StringUtil.nullConvert(accTransactionRes.getOppAccNo());
                    String oppAccName = StringUtil.nullConvert(accTransactionRes.getOppAccName());
                    String oppAccBank = StringUtil.nullConvert(accTransactionRes.getOppAccBank());
                    String transTime = StringUtil.nullConvert(accTransactionRes.getTransTime());
                    //这个是唯一的
                    String serialId = StringUtil.nullConvert(StringUtil.replaceBlank(accTransactionRes.getSerialId()));
                    String amt = StringUtil.nullConvert(accTransactionRes.getAmt());
                    String abs = StringUtil.nullConvert(accTransactionRes.getAbs());
                    String bankType = StringUtil.nullConvert(accTransactionRes.getBankType());
                    String serialNo = StringUtil.nullConvert(accTransactionRes.getSerialNo());
                    String currency = StringUtil.nullConvert(CurrencyEnum.getNameByCode(accTransactionRes.getCurCode()));
                    String uses = StringUtil.nullConvert(accTransactionRes.getUses());
                    //String curCode = StringUtil.nullConvert(accTransactionRes.getCurCode());
                    //这里转换成货币符号
                    String curCode = StringUtil.nullConvert(CurrencyEnum.getSymbolByCode(accTransactionRes.getCurCode()));
                    String postScript = StringUtil.nullConvert(accTransactionRes.getPostScript());
                    String data = StrUtil.format(getAccLogXmlTemplate(),bankAcc,bankName,accName,oppAccNo,oppAccName,oppAccBank,transTime,serialId,amt,abs,bankType,serialNo,currency,uses,postScript,curCode);
                    JSONObject paramsTemplate = new JSONObject();
                    paramsTemplate.put("loginName", "oa1");
                    paramsTemplate.put("dataXml", data);
                    Map<String, String> header =  MapBuilder.<String, String>create(true)
                            .put("apiCode", "8000340006")
                            .put("publicKey","ZJYAA9Z/jWMe1rC+Sjh8NVMRtwVMb5Bn0DbmIEsgpVlR3C9Ce7eDIk+3zDUT+v578prj")
                            .put("secretKey","B8sgQwEaskwM7BQj1XLvu3kdKD3xm7Lyd4FWsPaFQk5v+UtQLCkzVzLlCZ9Bdyv2j3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=")
                            .put("appId","800034")
                            .build();
                    String res = HttpRequest.post("http://127.0.0.1:8901/kangarooDataCenterV3/entranceController/externalCallInterface").addHeaders(header).body(paramsTemplate.toString()).timeout(60000).execute().body();
                    System.out.println("保存oa底表结果:"+res);
                }
            }
        }
    }

    /**
     * 查询OA附件，并上传到宁波银行
     *
     * @param oaId oa主键id
     * @return 宁波银行附件上传编码
     */
    @Override
    public String fileUpload(String oaId) {
        if (StrUtil.isNotEmpty(oaId)){
            try {
                //查询oa单据关联附件
                PayBillEntity oaBillEntity = new PayBillEntity();
                oaBillEntity.setDataSourceCode(oa_data_source_code);
                String sql = "	SELECT\n" +
                        "	formmain_0058.id,\n" +
                        "	formmain_0058.field0024 ,\n" +
                        "	CTP_ATTACHMENT.FILE_URL,\n" +
                        "	CTP_ATTACHMENT.FILENAME\n" +
                        "FROM\n" +
                        "	formmain_0058 \n" +
                        "	LEFT JOIN CTP_ATTACHMENT ON CTP_ATTACHMENT.SUB_REFERENCE = formmain_0058.field0024 \n" +
                        "WHERE\n" +
                        "	formmain_0058.field0061 = '境外'\n" +
                        "	and formmain_0058.id = '" + oaId+"'";
                oaBillEntity.setSql(sql);
                List<HashMap<String, Object>> dataList = payBillDao.getOaBillData(sql, oaBillEntity);
                if (CollectionUtils.isNotEmpty(dataList)){
                    HashMap<String, Object> oaDataMap =  dataList.get(0);
                    String fileUrl = String.valueOf(oaDataMap.get("FILE_URL"));
                    String fileName = String.valueOf(oaDataMap.get("FILENAME"));
                    if (StrUtil.isNotEmpty(fileUrl)){
                        //下载OA附件
                        byte[] bytes = restUtil.downloadFileBytes("oa1", "8000340000", fileUrl, fileName);
                        //上传到宁波银行
                        logger.info("上传附件到宁波银行:{},{}",fileUrl,fileName);
                        JSONObject param = new JSONObject();
                        param.put("fileName",fileName);
                        param.put("bytes",bytes);
                        param.put("channelNo","open-filegateway");
                        String res = executeEsb(param.toString(), "8000360008","800036");
                        JsonResultEntity jsonResultEntity = JSONObject.parseObject(res, JsonResultEntity.class);
                        if (null != jsonResultEntity && jsonResultEntity.isFlag()){
                            JSONObject attribute = JSONObject.parseObject(String.valueOf(jsonResultEntity.getAttribute()));
                            JSONObject data = attribute.getJSONObject("Data");
                            String fileCode = data.getString("data");
                            return fileCode;
                        }
                    }
                }
            }catch(Exception e){
                logger.error("上传附件到宁波银行出错:{}",e);
            }
        }
        return null;
    }

    /**
     * 更新u8c单据状态
     * @param main
     */
    @Override
    public void updateU8CStatus(JSONObject main,String integration_task_living_details_id) {
        String billCode = main.getString("serialNo");
        String dwbm = main.getString("dwbm");
        String state = main.getString("state");
        String userCode = main.getString("userCode");
        String id = main.getString("id");
        if (StrUtil.isNotEmpty(billCode)){
            PayBillEntity oaBillEntity = new PayBillEntity();
            oaBillEntity.setDataSourceCode(oa_data_source_code);
            String str = "update formmain_0058 set field0072 = 'N' where id = "+id;
            try {
                JSONObject param = new JSONObject();
                param.put("billno",billCode);
                param.put("corpcode",dwbm);
                param.put("billType","paybill");
                param.put("userCode",userCode);
                param.put("ckeckResult","3".equals(state) ? "Y" :"N");
                param.put("ckeckNote",getStateName(state));
                String result = executeEsb(param.toString(), "8000350060","800035");
                logger.info("调用U8C付款单审批返回参数:{}",result);
                //保存日志
                IntegrationTaskLivingDetailsEntity taskLivingDetail = new IntegrationTaskLivingDetailsEntity();
                taskLivingDetail.setCreate_time(new Date());
                taskLivingDetail.setModify_time(new Date());
                taskLivingDetail.setRootAppPk(id);
                taskLivingDetail.setRootAppBill(main.getString("serialNo"));
                taskLivingDetail.setPluginId("PayReqSyncU8CPlugin");
                taskLivingDetail.setRootAppNewData(param.toString());
                taskLivingDetail.setNewTransmitInfo(result);
                taskLivingDetail.setNewPushDate(new Date());
                JSONObject resultObj = JSON.parseObject(result);
                boolean flag = resultObj.getBoolean("flag");
                //保存日志
                saveLog(integration_task_living_details_id,flag,taskLivingDetail);
                //更新oa表单推送状态
                if (flag){
                    str = "update formmain_0058 set field0072 = 'Y' where id = "+id;
                }
            }catch (Exception e){
                logger.error("执行U8C付款单审批接口出错:{}",e);
            }
            //修改推送状态，防止重复推送
            oaBillEntity.setSql(str);
            try {
                payBillDao.updateStateForId(str,oaBillEntity);
            } catch (Exception e) {
                logger.error("执行U8C付款单审批接口后更新OA单据状态出错:{}",e);
            }
        }
    }
    //审批状态对应的中文
    private String getStateName(String state){
        if ("3".equals(state)){
            return "审批通过";
        }else if ("1".equals(state)){
            return "流程终止";
        }else if ("2".equals(state)){
            return "流程撤销";
        }
        return "审批通过";
    }
    public void saveLog(String integration_task_living_details_id, Boolean flag, IntegrationTaskLivingDetailsEntity taskLivingDetail){
        try {
            //判断，成功调用这个方法
            if (StrUtil.isEmpty(integration_task_living_details_id)){
                if (flag){
                    taskLivingDetailsService.saveLogToSuccess(taskLivingDetail);
                }else {
                    //失败 调用这个方法
                    taskLivingDetailsService.saveLogToFail(taskLivingDetail);
                }
            }else {
                taskLivingDetail.setId(integration_task_living_details_id);
                if (flag){
                    //如果是重试 成功调这个方法
                    taskLivingDetailsService.saveLogFailToSuccess(taskLivingDetail);
                }else {
                    //如果是重试 失败调这个方法
                    taskLivingDetailsService.updateLogFailToSuccess(taskLivingDetail);
                }
            }
        }catch (Exception e){
            logger.error("保存日志出错:{}",e);
        }
    }
    private static String getAccLogXmlTemplate(){
      //  return "<forms  version=\"2.1\"><formExport><summary  id=\"1828182906863371818\" name=\"formmain_0044\"/><definitions><column  id=\"field0001\" type=\"0\" name=\"我方账号\" isNullable=\"false\" length=\"200\"/><column  id=\"field0017\" type=\"0\" name=\"我方开户行\" isNullable=\"false\" length=\"200\"/><column  id=\"field0024\" type=\"0\" name=\"我方户名\" isNullable=\"false\" length=\"200\"/><column  id=\"field0029\" type=\"0\" name=\"客户方账号\" isNullable=\"false\" length=\"200\"/><column  id=\"field0022\" type=\"0\" name=\"客户方户名\" isNullable=\"false\" length=\"2000\"/><column  id=\"field0030\" type=\"0\" name=\"客户方开户行\" isNullable=\"false\" length=\"500\"/><column  id=\"field0018\" type=\"2\" name=\"交易日期\" isNullable=\"false\" length=\"255\"/><column  id=\"field0019\" type=\"0\" name=\"交易流水号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0020\" type=\"4\" name=\"收款金额\" isNullable=\"false\" length=\"20\"/><column  id=\"field0021\" type=\"0\" name=\"摘要\" isNullable=\"false\" length=\"500\"/><column  id=\"field0025\" type=\"0\" name=\"银行类型\" isNullable=\"false\" length=\"100\"/><column  id=\"field0026\" type=\"0\" name=\"银行流水号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0027\" type=\"0\" name=\"币种\" isNullable=\"false\" length=\"100\"/><column  id=\"field0028\" type=\"0\" name=\"用途\" isNullable=\"false\" length=\"800\"/><column  id=\"field0031\" type=\"0\" name=\"关联客户号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0032\" type=\"0\" name=\"客商编号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0033\" type=\"0\" name=\"交易代码\" isNullable=\"false\" length=\"100\"/><column  id=\"field0034\" type=\"0\" name=\"客商名称\" isNullable=\"false\" length=\"100\"/><column  id=\"field0035\" type=\"0\" name=\"对账码\" isNullable=\"false\" length=\"100\"/><column  id=\"field0036\" type=\"0\" name=\"备注\" isNullable=\"false\" length=\"800\"/><column  id=\"field0037\" type=\"0\" name=\"ERP业务参考号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0038\" type=\"0\" name=\"币种ID\" isNullable=\"false\" length=\"100\"/><column  id=\"field0039\" type=\"0\" name=\"币种编码\" isNullable=\"false\" length=\"100\"/><column  id=\"field0040\" type=\"0\" name=\"控制只关联1次\" isNullable=\"false\" length=\"100\"/><column  id=\"field0044\" type=\"4\" name=\"收款剩余金额\" isNullable=\"false\" length=\"20\"/><column  id=\"field0045\" type=\"0\" name=\"收款完成情况\" isNullable=\"false\" length=\"100\"/><column  id=\"field0046\" type=\"4\" name=\"是否收款完成\" isNullable=\"false\" length=\"20\"/></definitions><values><column  name=\"我方账号\"><value><![CDATA[{}]]></value></column><column  name=\"我方开户行\"><value><![CDATA[{}]]></value></column><column  name=\"我方户名\"><value><![CDATA[{}]]></value></column><column  name=\"客户方账号\"><value><![CDATA[{}]]></value></column><column  name=\"客户方户名\"><value><![CDATA[{}]]></value></column><column  name=\"客户方开户行\"><value><![CDATA[{}]]></value></column><column  name=\"交易日期\"><value><![CDATA[{}]]></value></column><column  name=\"交易流水号\"><value><![CDATA[{}]]></value></column><column  name=\"收款金额\"><value><![CDATA[{}]]></value></column><column  name=\"摘要\"><value><![CDATA[{}]]></value></column><column  name=\"银行类型\"><value><![CDATA[{}]]></value></column><column  name=\"银行流水号\"><value><![CDATA[{}]]></value></column><column  name=\"币种\"><value><![CDATA[{}]]></value></column><column  name=\"用途\"><value><![CDATA[{}]]></value></column><column  name=\"关联客户号\"><value><![CDATA[{}]]></value></column><column  name=\"客商编号\"><value><![CDATA[{}]]></value></column><column  name=\"交易代码\"><value><![CDATA[{}]]></value></column><column  name=\"客商名称\"><value><![CDATA[{}]]></value></column><column  name=\"对账码\"><value><![CDATA[{}]]></value></column><column  name=\"备注\"><value><![CDATA[{}]]></value></column><column  name=\"ERP业务参考号\"><value><![CDATA[{}]]></value></column><column  name=\"币种ID\"><value><![CDATA[{}]]></value></column><column  name=\"币种编码\"><value><![CDATA[{}]]></value></column><column  name=\"控制只关联1次\"><value><![CDATA[{}]]></value></column><column  name=\"收款剩余金额\"><value><![CDATA[{}]]></value></column><column  name=\"收款完成情况\"><value><![CDATA[{}]]></value></column><column  name=\"是否收款完成\"><value><![CDATA[{}]]></value></column></values><subForms/></formExport></forms>";
        return "<forms  version=\"2.1\"><formExport><summary  id=\"1828182906863371818\" name=\"formmain_0044\"/><definitions><column  id=\"field0001\" type=\"0\" name=\"我方账号\" isNullable=\"false\" length=\"200\"/><column  id=\"field0017\" type=\"0\" name=\"我方开户行\" isNullable=\"false\" length=\"200\"/><column  id=\"field0024\" type=\"0\" name=\"我方户名\" isNullable=\"false\" length=\"200\"/><column  id=\"field0029\" type=\"0\" name=\"客户方账号\" isNullable=\"false\" length=\"200\"/><column  id=\"field0022\" type=\"0\" name=\"客户方户名\" isNullable=\"false\" length=\"2000\"/><column  id=\"field0030\" type=\"0\" name=\"客户方开户行\" isNullable=\"false\" length=\"500\"/><column  id=\"field0018\" type=\"2\" name=\"交易日期\" isNullable=\"false\" length=\"255\"/><column  id=\"field0019\" type=\"0\" name=\"交易流水号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0020\" type=\"4\" name=\"收款金额\" isNullable=\"false\" length=\"20\"/><column  id=\"field0021\" type=\"0\" name=\"摘要\" isNullable=\"false\" length=\"500\"/><column  id=\"field0025\" type=\"0\" name=\"银行类型\" isNullable=\"false\" length=\"100\"/><column  id=\"field0026\" type=\"0\" name=\"银行流水号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0027\" type=\"0\" name=\"币种\" isNullable=\"false\" length=\"100\"/><column  id=\"field0028\" type=\"0\" name=\"用途\" isNullable=\"false\" length=\"800\"/><column  id=\"field0036\" type=\"0\" name=\"备注\" isNullable=\"false\" length=\"800\"/><column  id=\"field0039\" type=\"0\" name=\"币种编码\" isNullable=\"false\" length=\"100\"/></definitions><values><column  name=\"我方账号\"><value><![CDATA[{}]]></value></column><column  name=\"我方开户行\"><value><![CDATA[{}]]></value></column><column  name=\"我方户名\"><value><![CDATA[{}]]></value></column><column  name=\"客户方账号\"><value><![CDATA[{}]]></value></column><column  name=\"客户方户名\"><value><![CDATA[{}]]></value></column><column  name=\"客户方开户行\"><value><![CDATA[{}]]></value></column><column  name=\"交易日期\"><value><![CDATA[{}]]></value></column><column  name=\"交易流水号\"><value><![CDATA[{}]]></value></column><column  name=\"收款金额\"><value><![CDATA[{}]]></value></column><column  name=\"摘要\"><value><![CDATA[{}]]></value></column><column  name=\"银行类型\"><value><![CDATA[{}]]></value></column><column  name=\"银行流水号\"><value><![CDATA[{}]]></value></column><column  name=\"币种\"><value><![CDATA[{}]]></value></column><column  name=\"用途\"><value><![CDATA[{}]]></value></column><column  name=\"备注\"><value><![CDATA[{}]]></value></column><column  name=\"币种编码\"><value><![CDATA[{}]]></value></column></values><subForms/></formExport></forms>";
    }
}
