package com.hzya.frame.plugin.kjs.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapBuilder;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.plugin.kjs.service.IDeliveryOrderPluginService;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.service.IIntegrationTaskLivingDetailsService;
import com.hzya.frame.u8.delivery.dto.DeliveryDetailDTO;
import com.hzya.frame.u8.delivery.dto.DeliveryOrderDTO;
import com.hzya.frame.u8.delivery.entity.DeliveryDetailEntity;
import com.hzya.frame.u8.delivery.entity.DeliveryOrderEntity;
import com.hzya.frame.u8.delivery.service.DeliveryMapper;
import com.hzya.frame.u8.delivery.service.IDeliveryDetaiService;
import com.hzya.frame.u8.delivery.service.IDeliveryOrderService;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description 发货单插件service
 * @Author xiangerlin
 * @Date 2024/8/22 16:46
 **/
public class DeliveryOrderPluginServiceImpl extends KjsPluginBaseService implements IDeliveryOrderPluginService {
    Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private IDeliveryOrderService deliveryOrderService;
    @Autowired
    private IDeliveryDetaiService deliveryDetaiService;
    @Resource
    private IIntegrationTaskLivingDetailsService taskLivingDetailsService;

    /**
     * 发货单同步到wsm
     * @param requestJson
     * @return
     */
    @Override
    public String sync2wms(JSONObject requestJson) {
        if (null != requestJson){
            requestJson = super.check(requestJson);
            //数据源编码
            String datasourceCode = requestJson.getString("sourceCode");
            //开始时间
            String startTime = DateUtil.format(requestJson.getDate("startTime"),"yyyy-MM-dd HH:mm:ss");
            //结束时间
            String endTime = DateUtil.format(requestJson.getDate("endTime"),"yyyy-MM-dd HH:mm:ss");
            //1 查询发货单主表
            DeliveryOrderEntity orderEntity =  new DeliveryOrderEntity();
            orderEntity.setDataSourceCode(datasourceCode);
            String id = requestJson.getString("integration_task_living_details_id");
            if (StrUtil.isEmpty(id)){
                orderEntity.setStartTime(startTime);
                orderEntity.setEndTime(endTime);
                //orderEntity.setDlid("1000000152");
                orderEntity.setbReturnFlag("0");
            }else {
                IntegrationTaskLivingDetailsEntity taskLivingDetails = taskLivingDetailsService.get(id);
                orderEntity.setDlid(taskLivingDetails.getRootAppPk());
            }
            List<DeliveryOrderEntity> orderList = deliveryOrderService.query(orderEntity);
            if (CollectionUtils.isNotEmpty(orderList)){
                for (DeliveryOrderEntity order : orderList) {
                    try {
                        //2 查询发货单明细表
                        DeliveryDetailEntity detailEntity = new DeliveryDetailEntity();
                        detailEntity.setDlid(order.getDlid());
                        detailEntity.setDataSourceCode(datasourceCode);
                        List<DeliveryDetailEntity> detailList =  deliveryDetaiService.query(detailEntity);
                        //发货单表头entity转dto
                        DeliveryOrderDTO deliveryOrderDTO = DeliveryMapper.INSTANCE.deliveryEntityToDto(order);
                        //发货单表体entity转dto
                        List<DeliveryDetailDTO> deliveryDetailList = DeliveryMapper.INSTANCE.deliveryDetailListToDto(detailList);
                        deliveryOrderDTO.setItems(deliveryDetailList);
                        String reqParams = JSONObject.toJSONString(deliveryOrderDTO);
                        logger.info("发货单请求参数:{}",reqParams);
                        //调用wms接口
                        Map<String, String> headerMap = MapBuilder.<String, String>create(true)
                                .put("apiCode", "8000080008")
                                .put("publicKey","ZJYAWb7lhAUTYqekPkU+uHJv1/ObJxb7dT7sD8HPRDGAgyhCe7eDIk+3zDUT+v578prj")
                                .put("secretKey","fviZnLBsQUAGF8w8FSOdJi7XlIm/XAZclMxRagDLfTyJFlvnIBF3w66Hrpfzs8cYj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=")
                                .put("appId","800008")
                                .build();
                        String body = HttpRequest.post("http://127.0.0.1:9999/kangarooDataCenterV3/entranceController/externalCallInterface").addHeaders(headerMap).body(reqParams).timeout(60000).execute().body();
                        //保存自定义项  标记推送状态
                        order.setDataSourceCode(datasourceCode);
                       //deliveryOrderService.saveExtradefine(order);
                        if (StrUtil.isNotEmpty(body)){
                            JSONObject jsonObject = JSONObject.parseObject(body);
                            Boolean flag = jsonObject.getBoolean("flag");
                            //保存日志
                            IntegrationTaskLivingDetailsEntity taskLivingDetail = new IntegrationTaskLivingDetailsEntity();
                            taskLivingDetail.setCreate_time(new Date());
                            taskLivingDetail.setModify_time(new Date());
                            taskLivingDetail.setRootAppPk(order.getDlid());
                            taskLivingDetail.setRootAppBill(order.getCdlcode());
                            taskLivingDetail.setPluginId("kjsDeliveryOrderPlugin");
                            taskLivingDetail.setRootAppNewData(reqParams);
                            taskLivingDetail.setNewTransmitInfo(body);
                            taskLivingDetail.setNewPushDate(new Date());
                            this.saveLog(id,flag,taskLivingDetail);
                        }
                    }catch(Exception e){
                        logger.error("同步发货单到wsm出错:{}",e);
                    }
                }
            }
        }

        return null;
    }
}
