package com.hzya.frame.plugin.kjs.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapBuilder;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.plugin.kjs.service.IMomOrderPluginService;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.service.IIntegrationTaskLivingDetailsService;
import com.hzya.frame.u8.production.dto.MomOrderDTO;
import com.hzya.frame.u8.production.dto.MomOrderDetailDTO;
import com.hzya.frame.u8.production.entity.MomOrderDetailEntity;
import com.hzya.frame.u8.production.entity.MomOrderEntity;
import com.hzya.frame.u8.production.service.IMomOrderDetailService;
import com.hzya.frame.u8.production.service.IMomOrderService;
import com.hzya.frame.u8.production.service.MomOrderMapper;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description 生产订单同步wsm
 * @Author xiangerlin
 * @Date 2024/8/23 09:21
 **/
public class MomOrderPluginServiceImpl extends KjsPluginBaseService implements IMomOrderPluginService {
    Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private IMomOrderService momOrderService;
    @Autowired
    private IMomOrderDetailService momOrderDetailService;
    @Resource
    private IIntegrationTaskLivingDetailsService taskLivingDetailsService;
    /**
     * 生产订单单同步到wms
     *
     * @param requestJson
     * @return
     */
    @Override
    public String sync2wms(JSONObject requestJson) {
        if (null != requestJson){
            requestJson = super.check(requestJson);
            //数据源编码
            String datasourceCode = requestJson.getString("sourceCode");
            //开始时间
            String startTime = DateUtil.format(requestJson.getDate("startTime"),"yyyy-MM-dd HH:mm:ss");
            //结束时间
            String endTime = DateUtil.format(requestJson.getDate("endTime"),"yyyy-MM-dd HH:mm:ss");

            MomOrderEntity entity = new MomOrderEntity();
            entity.setDataSourceCode(datasourceCode);
            String id = requestJson.getString("integration_task_living_details_id");
            //如果这个id不为空说明是重试
            if (StrUtil.isEmpty(id)){
                entity.setStartTime(startTime);
                entity.setEndTime(endTime);
            }else {
                IntegrationTaskLivingDetailsEntity taskLivingDetails = taskLivingDetailsService.get(id);
                entity.setMoId(taskLivingDetails.getRootAppPk());
            }
            //1查询主表
            List<MomOrderEntity> orderList = momOrderService.query(entity);
            if (CollectionUtils.isNotEmpty(orderList)){
                for (MomOrderEntity order : orderList) {
                    try {
                        //查询生产订单明细表
                        MomOrderDetailEntity detailEntity = new MomOrderDetailEntity();
                        detailEntity.setMoId(order.getMoId());
                        if (StrUtil.isEmpty(id)){
                            detailEntity.setStartTime(startTime);
                            detailEntity.setEndTime(endTime);
                        }
                        detailEntity.setDataSourceCode(datasourceCode);
                        List<MomOrderDetailEntity> detailList = momOrderDetailService.query(detailEntity);
                        //表头转换dto
                        MomOrderDTO momOrderDTO = MomOrderMapper.INSTANCE.momOrderEntityToDto(order);
                        //表体转换dto
                        List<MomOrderDetailDTO> momOrderDetailList = MomOrderMapper.INSTANCE.momOrderDetailListToDto(detailList);
                        momOrderDTO.setItems(momOrderDetailList);
                        String reqParam = JSONObject.toJSONString(momOrderDTO);
                        logger.info("u8生产订单传wms请求参数:{}",reqParam);
                        Map<String, String> headerMap = MapBuilder.<String, String>create(true)
                                .put("apiCode", "8000080007")
                                .put("publicKey","ZJYAWb7lhAUTYqekPkU+uHJv1/ObJxb7dT7sD8HPRDGAgyhCe7eDIk+3zDUT+v578prj")
                                .put("secretKey","fviZnLBsQUAGF8w8FSOdJi7XlIm/XAZclMxRagDLfTyJFlvnIBF3w66Hrpfzs8cYj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=")
                                .put("appId","800008")
                                .build();
                        String body = HttpRequest.post("http://127.0.0.1:9999/kangarooDataCenterV3/entranceController/externalCallInterface").addHeaders(headerMap).body(reqParam).timeout(60000).execute().body();
                        if (StrUtil.isNotEmpty(body)){
                            JSONObject jsonObject = JSONObject.parseObject(body);
                            Boolean flag = jsonObject.getBoolean("flag");
                            //保存自定义项 标记推送状态
                            order.setDataSourceCode(datasourceCode);
                            //momOrderService.saveExtradefine(order);
                            //保存日志
                            IntegrationTaskLivingDetailsEntity taskLivingDetail = new IntegrationTaskLivingDetailsEntity();
                            taskLivingDetail.setCreate_time(new Date());
                            taskLivingDetail.setModify_time(new Date());
                            taskLivingDetail.setRootAppPk(order.getMoId());
                            taskLivingDetail.setRootAppBill(order.getMocode());
                            taskLivingDetail.setPluginId("kjsMomOrderPlugin");
                            taskLivingDetail.setRootAppNewData(reqParam);
                            taskLivingDetail.setNewTransmitInfo(body);
                            taskLivingDetail.setNewPushDate(new Date());
                            this.saveLog(id,flag,taskLivingDetail);
                        }
                    }catch (Exception e){
                        logger.error("生产订单同步wsm出错:{}",e);
                    }
                }
            }
        }
        return null;
    }
}
