package com.hzya.frame.ningboBankTreasury.enums;

/**
 * @Description T/T汇款 支付状态枚举
 * @Author xiangerlin
 * @Date 2024/8/2 15:42
 **/
public enum TTPayStatusEnum{

    // 成功和失败状态
    SUCCESS("0", "交易成功"),
    TRANSACTION_FAILED("2", "交易失败"),
    TRANSACTION_SENDING("8", "交易发送中"),

    // 审批状态

    APPROVAL_IN_PROGRESS("11-94", "审批中"),
    APPROVAL_PASSED("95", "审批通过"),
    APPROVAL_REJECTED_CLOSED("-3", "审批打回，交易关闭"),

    // 其他状态
    TEMPORARY_STORAGE("7", "暂存"),
    DELETED("-2", "删除"),
    BANK_PROCESSING("1", "银行处理中"),
    APPROVAL_REJECTED("-1", "审批打回");

    // 枚举属性
    private final String code;
    private final String description;

    // 枚举构造函数
    TTPayStatusEnum(String code, String description) {
        this.code =  code;
        this.description = description;
    }

    // 获取状态码
    public String getCode() {
        return code;
    }

    // 获取状态描述
    public String getDescription() {
        return description;
    }


    // 根据状态码查找枚举
    public static TTPayStatusEnum fromCode(String code) {
        for (TTPayStatusEnum status : TTPayStatusEnum.values()) {
            if (status.getCode().equals(code)) {
                return status;
            }
        }
        throw new IllegalArgumentException("Unknown code: " + code);
    }

    public static String getByCode(String code){
        for (TTPayStatusEnum status : TTPayStatusEnum.values()) {
            if (status.getCode().equals(code)) {
                return status.getDescription();
            }
        }
        throw new IllegalArgumentException("Unknown code: " + code);
    }

    /**
     * 是否为最终态
     * @param code
     * @return
     */
    public static boolean isFinally(String code){
        return SUCCESS.code.equals(code)
                || APPROVAL_REJECTED_CLOSED.code.equals(code)
                || DELETED.code.equals(code)
                || TRANSACTION_FAILED.code.equals(code);
    }
}
