package com.hzya.frame.ningboBankTreasury.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.ningboBankTreasury.dto.req.AccTransactionReq;
import com.hzya.frame.ningboBankTreasury.dto.res.AccTransactionRes;
import com.hzya.frame.ningboBankTreasury.dto.res.AccTrxResponse;
import com.hzya.frame.ningboBankTreasury.service.INingboBankTreasuryService;
import com.hzya.frame.ningboBankTreasury.util.NingboBankUtil;
import com.hzya.frame.sysnew.application.entity.SysExtensionApiEntity;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import com.nbcb.sdk.OpenSDK;
import com.nbcb.sdk.aes.exception.SDKException;
import com.nbcb.sdk.aes.param.ConfigParam;
import com.nbcb.sdk.file.FileDownloadResponse;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Service(value = "ningboBankTreasuryServiceImpl")
public class NingboBankTreasuryServiceImpl implements INingboBankTreasuryService {
    private static final Logger logger = LoggerFactory.getLogger(NingboBankTreasuryServiceImpl.class);


    /**
     * @param jsonObject
     * @return com.hzya.frame.web.entity.JsonResultEntity
     * @Author lvleigang
     * @Description 单笔转账查证接口
     * @Date 4:45 下午 2024/7/9
     **/
    @Override
    public Object sendNbBank(JSONObject jsonObject) {
        JSONObject returnJson = new JSONObject();
        JSONObject entity = getstrObj("jsonStr", jsonObject);
        String dataJson = entity.getString("Data");
        String appKey = entity.getString("appKey");
        String privateKey = entity.getString("privateKey");
        String publicUrl = entity.getString("publicUrl");
        String publicKey = entity.getString("publicKey");
        String productID = entity.getString("productID");
        String serviceID = entity.getString("serviceID");
        //初始化宁波银行OpenSDK
        try {
            ConfigParam configParam = new ConfigParam(privateKey,publicUrl,appKey,publicKey,10000,20000);
            OpenSDK.dynamicInit(configParam,true);
        } catch (SDKException e) {
            logger.error("初始化宁波银行OpenSDK错误:{}",e.getMessage());
            returnJson.put("retCode","9999");
            returnJson.put("retMsg","初始化宁波银行OpenSDK错误");
            return returnJson;
        }
        //发送数据
        try {
            logger.info("调用宁波银行接口最终参数：{}",dataJson);
            String returnData = OpenSDK.send(productID,serviceID,dataJson.replace(" ",""));
            if(returnData == null || "".equals(returnData)){
                returnJson.put("retCode","9999");
                returnJson.put("retMsg","发送宁波银行OpenSDK返回错误");
                return returnJson;
            }
            try {
                returnJson = JSONObject.parseObject(returnData);
                JSONObject retData = returnJson.getJSONObject("Data");
                String retCode = retData.getString("retCode");
                String retMsg = retData.getString("retMsg");
                returnJson.put("retCode",retCode);
                returnJson.put("retMsg",retMsg);
                return returnJson;
            } catch (Exception e) {
                returnJson.put("retCode","9999");
                returnJson.put("retMsg","发送宁波银行OpenSDK返回转换Json错误");
                return returnJson;
            }
        } catch (Exception e) {
            returnJson.put("retCode","9999");
            returnJson.put("retMsg","发送宁波银行OpenSDK错误");
            return returnJson;
        }
    }



    /**
     * @param jsonObject
     * @return com.hzya.frame.web.entity.JsonResultEntity
     * @Author lvleigang
     * @Description 发送宁波银行下载文件
     *
     * @Date 4:45 下午 2024/7/9
     **/
    @Override
    public Object sendNbBankFileDownload(JSONObject jsonObject) {
        JSONObject returnJson = new JSONObject();
        JSONObject entity = getstrObj("jsonStr", jsonObject);
        String dataJson = entity.getString("Data");
        JSONObject data = JSONObject.parseObject(dataJson);
        JSONObject dataFileid = data.getJSONObject("Data");
        String appKey = entity.getString("appKey");
        String privateKey = entity.getString("privateKey");
        String publicUrl = entity.getString("publicUrl");
        String publicKey = entity.getString("publicKey");
        String productID = entity.getString("productID");
        String serviceID = entity.getString("serviceID");
        //初始化宁波银行OpenSDK
        try {
            ConfigParam configParam = new ConfigParam(privateKey,publicUrl,appKey,publicKey,10000,20000);
            OpenSDK.dynamicInit(configParam,true);
        } catch (SDKException e) {
            logger.error("初始化宁波银行OpenSDK错误:{}",e.getMessage());
            returnJson.put("retCode","9999");
            returnJson.put("retMsg","初始化宁波银行OpenSDK错误");
            return returnJson;
        }
        //发送数据
        try {
            FileDownloadResponse returnData = OpenSDK.sendFileDownload(productID,"open-filegateway",dataFileid.getString("fileId"),null);
            if(returnData == null || "".equals(returnData)){
                returnJson.put("retCode","9999");
                returnJson.put("retMsg","发送宁波银行OpenSDK返回错误");
                return returnJson;
            }
            try {

                //returnJson = JSONObject.parseObject(returnData);
                //JSONObject retData = returnJson.getJSONObject("Data");
                String retCode = returnData.getRetCode();
                String retMsg = returnData.getRetMsg();
                returnJson.put("data",returnData.getData());
                returnJson.put("retCode",retCode);
                returnJson.put("retMsg",retMsg);
                return returnJson;
            } catch (Exception e) {
                returnJson.put("retCode","9999");
                returnJson.put("retMsg","发送宁波银行OpenSDK返回转换Json错误");
                return returnJson;
            }
        } catch (Exception e) {
            returnJson.put("retCode","9999");
            returnJson.put("retMsg","发送宁波银行OpenSDK错误");
            return returnJson;
        }
    }

    /**
     * 分页方式 全量查询境内账户交易明细
     * accTransactionReq.custId  客户号 必填
     * accTransactionReq.bankAccList  账号列表 必填
     *
     * @param accTransactionReq
     * @return
     */
    @Override
    public List<AccTransactionRes> queryAccDetails(AccTransactionReq accTransactionReq) {
       accTransactionReq = defValue(accTransactionReq);
        //当前页数
        int currentPage = Integer.valueOf(accTransactionReq.getCurrentPage());
        //总页数
        int totalPage = 0;
        JSONObject data = new JSONObject();
        List<AccTransactionRes> resList = new ArrayList<>();
        try {
            do {
                ConfigParam configParam = new ConfigParam("MIGTAgEAMBMGByqGSM49AgEGCCqBHM9VAYItBHkwdwIBAQQgTt6OadouxxxSDg/0SZdOIBqtnFGKrpwmL+quEUf+hfSgCgYIKoEcz1UBgi2hRANCAAQGHNN2NO7btp+w1248yk98E0asGC8sG0bmIzdom4M0xp3vc4IfXjmZ8omUz4oVcjzAuyMSSpJM6uPLcU1SH4HZ","http://cz-test.nbcb.com.cn:7070/nbcb/api","54b3e05c-7983-4e6c-8920-f9ab8a0004a6","MFkwEwYHKoZIzj0CAQYIKoEcz1UBgi0DQgAEXAt6fxeKlpREIyXoT3jDwGU7L6JkKrzdOKP1cMB14rtNwwlyaUGLrtuHVoArAHP3x7XN2rUw0GxSngYQwOAsjQ==",10000,20000);
                OpenSDK.dynamicInit(configParam,true);
                data.put("Data",JSONObject.toJSONString(accTransactionReq));
                logger.info("查询交易明细请求参数:{}",data.toString());
                String returnData = OpenSDK.send(NingboBankUtil.TMSREPORT, NingboBankUtil.QUERYACCDETAIL,data.toString());
                System.out.println(returnData);
                logger.info("查询交易明细返回数据:{}",returnData);
                AccTrxResponse accTrxResponse = AccTrxResponse.valueOf(returnData);
                if (null != accTrxResponse && NingboBankUtil.isSuccess(accTrxResponse.getRetCode())){
                    totalPage = Integer.valueOf(accTrxResponse.getTotalPage());
                    resList.addAll(accTrxResponse.getList());
                }
                currentPage++;//页码自增
                accTransactionReq.setCurrentPage(String.valueOf(currentPage));//页码
            }while (currentPage <= totalPage);//当前页 <= 总页数 继续循环
        } catch (Exception e) {
            logger.error("查询账户交易明细出错:{}",e);
        }
        return resList;
    }

    /**
     * esb方法
     * 分页方式 全量查询境内账户交易明细
     *
     * @param jsonObject
     * @return
     */
    @Override
    public JsonResultEntity queryAccountDetailsEsb(JSONObject jsonObject) {
        JSONObject entity = getstrObj("jsonStr", jsonObject);
        String dataJson = entity.getJSONObject("Data").getString("Data");
        AccTransactionReq accTransactionReq = JSONObject.parseObject(dataJson,AccTransactionReq.class );
        accTransactionReq = defValue(accTransactionReq);
        //当前页数
        int currentPage = Integer.valueOf(accTransactionReq.getCurrentPage());
        //总页数
        int totalPage = 0;
        JSONObject data = new JSONObject();
        List<AccTransactionRes> resList = new ArrayList<>();
        try {
            do {
                data.put("Data",JSONObject.toJSONString(accTransactionReq));
                entity.put("Data",data);
                JSONObject param = new JSONObject();
                param.put("jsonStr",entity);
                logger.info("查询交易明细请求参数:{}",JSONObject.toJSONString(param));
                JSONObject returnObj = (JSONObject) sendNbBank(param);
                logger.info("查询交易明细返回数据:{}",returnObj.toString());
                AccTrxResponse accTrxResponse = AccTrxResponse.valueOf(returnObj.toString());
                if (null != accTrxResponse && NingboBankUtil.isSuccess(accTrxResponse.getRetCode())){
                    totalPage = Integer.valueOf(accTrxResponse.getTotalPage());
                    resList.addAll(accTrxResponse.getList());
                }
                currentPage++;//页码自增
                accTransactionReq.setCurrentPage(String.valueOf(currentPage));//页码
            }while (currentPage <= totalPage);//当前页 <= 总页数 继续循环
        } catch (Exception e) {
            logger.error("查询账户交易明细出错:{}",e);
            return BaseResult.getFailureMessageEntity("查询账户交易明细出错"+e.getMessage());
        }
        return BaseResult.getSuccessMessageEntity("查询账户交易明细成功",resList);
    }

    //默认值检查
    private AccTransactionReq defValue(AccTransactionReq accTransactionReq){
        if (StrUtil.isEmpty(accTransactionReq.getPageSize())){
            accTransactionReq.setPageSize("1000");
        }
        if (StrUtil.isEmpty(accTransactionReq.getCurrentPage())){
            accTransactionReq.setCurrentPage("1");
        }
        if (StrUtil.isEmpty(accTransactionReq.getQueryType())){
            accTransactionReq.setQueryType("0");
        }
        if (StrUtil.isEmpty(accTransactionReq.getCdSign())){
            accTransactionReq.setCdSign(NingboBankUtil.SHOURU);
        }
        return accTransactionReq;
    }
    /**
     * @param entity
     * @content 此方法获取初始化参数，拼接请求参数
     * @author laborer
     * @date 2024/5/27 0027 11:38
     */
    @Override
    public SysExtensionApiEntity doChangeData(SysExtensionApiEntity entity) {
        try {
            if(entity == null ){
                return entity;
            }
            Map<String, String> returnHeaders = new HashMap<>();
            JSONObject returnDataData = new JSONObject();
            JSONObject returnData = new JSONObject();
            Map<String, String> headers = entity.getHeaders();
            String bodys = entity.getBodys();
            JSONObject data = JSONObject.parseObject(bodys);

            //先处理转发请求header
            returnHeaders.put("yatl",headers.get("yatl"));
            returnHeaders.put("yadj",headers.get("yadj"));

            returnDataData.put("appKey",headers.get("appKey"));
            returnDataData.put("privateKey",headers.get("privateKey"));
            returnDataData.put("publicUrl",headers.get("publicUrl"));
            returnDataData.put("publicKey",headers.get("publicKey"));
            returnDataData.put("productID",headers.get("productID"));
            returnDataData.put("serviceID",headers.get("serviceID"));
            data.put("custId",headers.get("custId"));
            returnData.put("Data",data);
            returnDataData.put("Data",returnData);
            entity.setHeaders(returnHeaders);
            entity.setBodys(returnDataData.toJSONString());
        } catch (Exception e) {
            //发送内部处理方法错误
            logger.error("宁波银行内部转换接口错误：{}",e.getMessage());
        }
        return entity;
    }




    public JSONObject getstrObj(String key, JSONObject object) {
        if (checkStr(object.getString(key)) ) {
            return object.getJSONObject(key);
        }
        return new JSONObject();
    }
    /**
     * @param str
     * @return void
     * @Author lvleigang
     * @Description 校验字符串
     * @Date 11:41 上午 2022/12/7
     **/
    protected Boolean checkStr(String str) {
        Boolean flag = true;
        if (str == null || "".equals(str)) {
            flag = false;
        }
        return flag;
    }
}
