package com.hzya.frame.u8.saleinv.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.basedao.service.impl.BaseService;
import com.hzya.frame.crm.util.CrmUtil;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.service.IIntegrationTaskLivingDetailsService;
import com.hzya.frame.u8.saleinv.dao.ISaleInvDao;
import com.hzya.frame.u8.saleinv.dao.ISaleInvDetailDao;
import com.hzya.frame.u8.saleinv.entity.SaleInvDetailEntity;
import com.hzya.frame.u8.saleinv.entity.SaleInvEntity;
import com.hzya.frame.u8.saleinv.service.ISaleInvService;

import com.hzya.frame.u8.saleout.entity.SaleOutDetailEntity;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 *
 * @content 销售发票
 * @className: Administrator
 * @author laborer
 * @date 2024-10-18 9:27
 *
 */
@Service(value = "SaleInvServiceImpl")
public class SaleInvServiceImpl extends BaseService<SaleInvEntity,String> implements ISaleInvService {
    Logger logger = LogManager.getLogger(getClass());
    @Autowired
    private ISaleInvDao saleInvDao;
    @Autowired
    private ISaleInvDetailDao saleInvDetailDao;
    @Resource
    private IIntegrationTaskLivingDetailsService taskLivingDetailsService;
    @Override
    public void getU8SaleInvToCrm(JSONObject requestJson) {
        //获取重推信息id，并记录日志
        JSONObject requestJsonObj = JSON.parseObject(requestJson.toString(),JSONObject.class);
        String interId = requestJsonObj.getString("integration_task_living_details_id");
        logger.info("U8销售发票单重推解析后："+requestJsonObj.toString());
        logger.info("U8销售发票单推送CRM重新执行的任务主键："+interId);

        //更换数据源
        JsonResultEntity resultEntity = new JsonResultEntity();
        SaleInvEntity entity = new SaleInvEntity();
        requestJson.put("db_code","YONYOUU8");
        entity.setDataSourceCode(requestJson.getString("db_code"));

        //根据重推信息id，查询该条重推记录信息
        IntegrationTaskLivingDetailsEntity oldMsg = new IntegrationTaskLivingDetailsEntity();
        if(StrUtil.isNotEmpty(interId)){
            oldMsg = taskLivingDetailsService.get(interId);
        }
        String rootAppPk = oldMsg.getRootAppPk();
        entity.setIds(rootAppPk);
        StrUtil.isEmpty(rootAppPk);//entity.setQueryState("查询");
        //如果是重推，是否通过entity传入的id，只能查到一条数据。
        //entity.setId("-8845102890648320822");
        List<SaleInvEntity> soleList = saleInvDao.getU8SaleInv(entity);
        logger.info("销售发票查询数据{}",JSONObject.toJSONString(soleList));
        if(CollectionUtils.isNotEmpty(soleList)){
            soleList.forEach(sale -> {
                try{
                    //获取明细数据进行拼接
                    String id = sale.getSBVID();
                    if(!StrUtil.isEmpty(id)){
                        SaleInvDetailEntity line = new SaleInvDetailEntity();
                        line.setSBVID(id);
                        line.setAccId(id);
                        line.setDataSourceCode(requestJson.getString("db_code"));
                        List<SaleInvDetailEntity> lineList = saleInvDetailDao.getU8InvDetail(line);
                        logger.info("销售发票单明细查询数据{}",JSONObject.toJSONString(lineList));
                        sale.setSaleInvDetailEntityList(lineList);
                        logger.info("组装数据");
                        JSONObject main = bindingAdd(sale);
                        logger.info("销售发票推送数据："+main.toString());
                        String result = CrmUtil.sendU8TOCrmEsb(main.toString(), "8000340003");
                        logger.info("销售发票推送结果："+result);
                        IntegrationTaskLivingDetailsEntity logDetails = new IntegrationTaskLivingDetailsEntity();
                        logDetails.setRootAppPk(sale.getIds());
                        logDetails.setRootAppBill(sale.getCsbvcode());
                        logDetails.setNewTransmitInfo(result);
                        logDetails.setNewPushDate(new Date());
                        logDetails.setRootAppNewData(JSON.toJSONString(sale));
                        logDetails.setPluginId("SaleConPlugin");

                        JSONObject resultObj = JSON.parseObject(result);
                        boolean flag = resultObj.getString("code").equals("0");
                        if(flag){
                            logDetails.setNewTransmitInfo("推送成功");
                            // 推送成功，更新状态
                            sale.setState("Y");
                        }else{
                            // 推送失败
                            sale.setState("N");
                        }
                        sale.setDataSourceCode(requestJson.getString("db_code"));
                        try{
                            saveLog(interId, logDetails, flag);
                        }catch (Exception e){
                            logger.error("保存日志失败"+e.getMessage());
                            saleInvDao.updateState(sale);
                            e.printStackTrace();
                        }
                        saleInvDao.updateState(sale);
                    }
                } catch (Exception e) {
                    logger.error("销售发票执行失败"+e.getMessage());
                    e.printStackTrace();
                }
            });

        }
    }
    /**
     *
     * @content 保存日志信息
     * @className: Administrator
     * @author laborer
     * @date 2024-10-18 11:27
     *
     */

    private  void saveLog(String interId, IntegrationTaskLivingDetailsEntity logDetails, boolean flag) throws Exception {
        if(StrUtil.isEmpty(interId)){
            if(flag){
                //首推成功
                taskLivingDetailsService.saveLogToSuccess(logDetails);
            }else{
                //首推失败
                taskLivingDetailsService.saveLogToFail(logDetails);
            }
        }else{
            logDetails.setId(interId);
            if(flag){
                //重推成功
                taskLivingDetailsService.saveLogFailToSuccess(logDetails);
            }else{
                //重推失败，把失败信息，更新到重推表中
                taskLivingDetailsService.updateLogFailToSuccess(logDetails);
            }
        }
    }

    /**
     *
     * @content 组装推送参数
     * @className: Administrator
     * @author laborer
     * @date 2024-10-18 11:13
     *
     */

    private JSONObject bindingAdd(SaleInvEntity sale) {
        JSONObject main = new JSONObject();
        main.put("AccId",sale.getAccId());
        main.put("billid",sale.getId());
        main.put("LoginDate", DateUtil.format(new Date(),"yyyy-MM-dd HH:mm:ss"));
        JSONObject head = new JSONObject();
        head.put("csbvcode",sale.getCsbvcode());
        head.put("ddate",sale.getDdate());
        head.put("syncFlag",sale.getSyncFlag());
        head.put("isum",sale.getIsum());
        JSONArray body = new JSONArray();
        main.put("head",head);
        List<SaleInvDetailEntity> lineList = sale.getSaleInvDetailEntityList();
        if(CollectionUtils.isNotEmpty(lineList)){
            lineList.forEach(line -> {
                JSONObject lineObj = new JSONObject();
                lineObj.put("cdlcode",line.getCdlcode());
                lineObj.put("idlsum",line.getIdlsum());
                lineObj.put("idlrowno",line.getIdlrowno());
                body.add(lineObj);
            });
        }
        main.put("body",body);
        return main;
    }
}
